import json
from base64 import urlsafe_b64encode

from django.conf import settings
from django.contrib.auth import get_user_model
from django.test import Client, TestCase
from django.urls import reverse
from djangoldp_webpushnotification.models import VAPIDKeyset
from ecdsa import NIST256p, SigningKey
from webpush.models import PushInformation, SubscriptionInfo


class TestSendWebPush(TestCase):
    def setUp(self):
        self.client = Client()
        self.user = get_user_model().objects.create(
            username="john", email="jlennon@beatles.com", password="glass onion"
        )

    def tearDown(self):
        self.user.delete()

    def gen_vapid_key(self):
        generated = SigningKey.generate(curve=NIST256p)
        encoded = urlsafe_b64encode(generated.to_string()).strip(b"=")
        return VAPIDKeyset.objects.create(_private_key=encoded)

    def test_no_matching_user(self):
        self.client.force_login(self.user)
        payload = {"head": "foo", "body": "bar", "id": 666}

        url = reverse("send_webpush")
        response = self.client.post(
            url, data=json.dumps(payload), content_type="application/json"
        )

        self.assertEqual(response.status_code, 404)

    def test_missing_vapid_key(self):
        self.client.force_login(self.user)
        payload = {"head": "foo", "body": "bar", "id": self.user.id}

        url = reverse("send_webpush")
        response = self.client.post(
            url, data=json.dumps(payload), content_type="application/json"
        )

        self.assertEqual(response.status_code, 400)

    def test_missing_vapid_email(self):
        self.client.force_login(self.user)

        self.gen_vapid_key()

        payload = {"head": "foo", "body": "bar", "id": self.user.id}

        del settings.WEBPUSH_SETTINGS["VAPID_ADMIN_EMAIL"]

        url = reverse("send_webpush")
        response = self.client.post(
            url, data=json.dumps(payload), content_type="application/json"
        )

        self.assertEqual(response.status_code, 400)

    def test_send_webpush(self):
        self.client.force_login(self.user)

        self.gen_vapid_key()

        payload = {"head": "foo", "body": "bar", "id": self.user.id}

        settings.WEBPUSH_SETTINGS["VAPID_ADMIN_EMAIL"] = "george@harrison.beatles"

        url = reverse("send_webpush")
        response = self.client.post(
            url, data=json.dumps(payload), content_type="application/json"
        )

        self.assertEqual(response.status_code, 200)
